=head1 NAME

package Mail::SpamAssassin::Plugin::sadwl

=head1 SYNOPSIS

  loadplugin Mail::SpamAssassin::Plugin::sadwl

  use_sadwl                  1
  sadwl_sql_dsn              DBI:mysql:sadwl:localhost
  sadwl_sql_username         sql_user
  sadwl_sql_password         sql_pass

=head1 DESCRIPTION

sadwl
Is a SpamAssassin Plugin that record into a table the couple :
[email address domain] + [sender relay domain]
A dynamyc score correction can be assigned to this

=cut
package Mail::SpamAssassin::Plugin::sadwl;

use strict;
use warnings;
use bytes;
use re 'taint';
use Errno qw(EBADF);

use Mail::Address;
use Mail::SpamAssassin::Plugin;
use Mail::SpamAssassin::Locales;
use Mail::SpamAssassin::Logger;
use Mail::SpamAssassin::Constants qw(:sa :ip);

use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin);

# constructor: register the eval rule
sub new {
  my $class = shift;
  my $mailsaobject = shift;

  # some boilerplate...
  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsaobject);
  bless ($self, $class);

  # the important bit!
  $self->set_config($mailsaobject->{conf});
  $self->register_eval_rule("sadwl_check");

  return $self;
}


# set_config
# ==========
# Load configuration parameters
sub set_config {
  my ($self, $conf) = @_;
  my @cmds = ();

  push (@cmds, {
        setting => 'use_sadwl',
        default => 1,
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_NUMERIC,
  });

  push (@cmds, {
        setting => 'sadwl_local_server_prefix',
        default => '10.',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'sadwl_sql_dsn',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'sadwl_sql_username',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  push (@cmds, {
        setting => 'sadwl_sql_password',
        type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING,
  });

  $conf->{parser}->register_commands(\@cmds);
  dbg("sadwl: main: Configuration Loaded");
}


# _get_relay_ext
# ==============
# Get relay subroutine
sub _get_relay_ext {
  # Sub to get sender IP address
  my ($self, $scanner) = @_;
  return $scanner->{relays_external}->[0];
}


# sadwl_check
# ===========
# Main Procedure
sub sadwl_check {
  my ($self, $pms) = @_;

  # Retrieving parameters
  my $use_sadwl = $self->{main}->{conf}->{use_sadwl};
  my $sadwl_local_server_prefix = $self->{main}->{conf}->{sadwl_local_server_prefix};
  my $sadwl_sql_dsn = $self->{main}->{conf}->{sadwl_sql_dsn};
  my $sadwl_sql_username = $self->{main}->{conf}->{sadwl_sql_username};
  my $sadwl_sql_password = $self->{main}->{conf}->{sadwl_sql_password};
  #dbg("sadwl: sadwl_check: Start");
  #dbg("sadwl: sadwl_check: use_sadwl: " . $use_sadwl);
  #dbg("sadwl: sadwl_check: sadwl_local_server_prefix: " . $sadwl_local_server_prefix);
  #dbg("sadwl: sadwl_check: sadwl_sql_dsn: " . $sadwl_sql_dsn);
  #dbg("sadwl: sadwl_check: sadwl_sql_username: " . $sadwl_sql_username);
  #dbg("sadwl: sadwl_check: sadwl_sql_password: " . $sadwl_sql_password);

  # Check if enabled and terminating if not
  if ($use_sadwl ne 1) {
   dbg("sadwl: sadwl_check: Skipped due use_sadwl not enabled");
   return 0;
  }

  # Find Sender Ip and exit if internal (sadwl_local_server_prefix)
  my $senderip = "";
  foreach my $rly  (@{$pms->{relays_trusted}}, @{$pms->{relays_untrusted}}) {
   if ($rly->{ip}) {$senderip = $rly->{ip}; last;}
  }
  dbg("sadwl: sadwl_check: Sender IP: " . $senderip);
  if (index($senderip, $sadwl_local_server_prefix) eq 0) {
   dbg("sadwl: sadwl_check: Local server prefix, skip");
   return 0;
  }

  # Get From and calculate From Domain
  my $from = lc $pms->get('From:addr');
  $from =~ s/^\s+|\s+$//g;
  dbg("sadwl: sadwl_check: From: " . $from);
  my $fromdomain = '-undefined-';
  if ( $from ne '' ) {
   my ($fromtest) = Mail::Address->parse($from);
   $fromdomain = $fromtest->host;
  }
  dbg("sadwl: sadwl_check: FromDomain: " . $fromdomain);

  # Get Envelope From and calculate Envelope From Domain
  my $envfrom = lc $pms->get('EnvelopeFrom:addr');
  $envfrom =~ s/^\s+|\s+$//g;
  dbg("sadwl: sadwl_check: EnvFrom: " . $envfrom);
  my $envfromdomain = '-undefined-';
  if ( $envfrom ne '' ) {
   my ($envfromtest) = Mail::Address->parse($envfrom);
   $envfromdomain = $envfromtest->host;
  }
  dbg("sadwl: sadwl_check: EnvFromDomain: " . $envfromdomain);

  # Find reverse
  my $res = Net::DNS::Resolver->new;
  my $reverse = "-undefined-";
  my $domreverse = "-undefined-";
  my $target_IP = join('.', reverse split(/\./, $senderip)).".in-addr.arpa";
  my $query = $res->query("$target_IP", "PTR");
  if ($query) {
   foreach my $rr ($query->answer) {
    next unless $rr->type eq "PTR";
    $reverse = $rr->rdatastr;
    $reverse =~ s/.\s*$//;
    #$domreverse = (split /\./, $reverse, 2)[1];
   }
  } else {
   dbg("sadwl: sadwl_check: Reverse failed, skipping " . $senderip);
   return 0;
  }
  dbg("sadwl: sadwl_check: Reverse: " . $reverse);
  #dbg("sadwl: sadwl_check: DomReverse original: " . $domreverse);
  # Transform reverse in 2nd level
  $domreverse = $reverse;
  $domreverse =~ s/^.+\.([^\.]+\.[^\.]+)$/$1/;
  dbg("sadwl: sadwl_check: DomReverse 2nd level: " . $domreverse);

  # Info from lashhop not useful
  # my $lasthop = $self->_get_relay_ext($pms);
  # my $hip = $lasthop->{ip};
  # my $hrdns = $lasthop->{rdns};
  # my $hhelo = $lasthop->{helo};
  # dbg("sadwl: sadwl_check: Lasthop IP: " . $hip);
  # dbg("sadwl: sadwl_check: Lasthop Rdns: " . $hrdns);
  # dbg("sadwl: sadwl_check: Lasthop Helo: " . $hhelo);

  # Connect DB
  my $dbh = DBI->connect($sadwl_sql_dsn, $sadwl_sql_username, $sadwl_sql_password, {'PrintError' => 0});
  my $sql="";
  my $sth="";
  my $vscore=0;
  my $rv="";

  # Table Mainteinance sometimes using random
  my $random = int rand 50;
  dbg("sadwl: sadwl_check: Random: " . $random);
  if ($random eq 1) {
   dbg("sadwl: sadwl_check: Execute table maintainance");
   # Delete if have only 1 entry and older that 10 days (only if no score assigned by user)
   $dbh->do("DELETE FROM sadwl WHERE date(last) < CURDATE() - interval 10 day  AND num = 1 AND score IS NULL;") or dbg("sadwl: sadwl_check: Database error: " .  $DBI::errstr );
   # Delete if have less than 10 entry and older that 30 days (only if no score assigned by user)
   $dbh->do("DELETE FROM sadwl WHERE date(last) < CURDATE() - interval 30 day  AND num < 10 AND score IS NULL;") or dbg("sadwl: sadwl_check: Database error: " .  $DBI::errstr );
  }

  # Insert/Update record into sadwl
  dbg("sadwl: sadwl: Inserting/Updating sadwl: " . $fromdomain . " " . $envfromdomain . " " . $domreverse);
  $sql  = "INSERT INTO sadwl SET last = NOW() ";
  $sql .= ", domain = '$fromdomain' ";
  $sql .= ", envdomain =  '$envfromdomain' ";
  $sql .= ", srv_dom_reverse = '$domreverse' ";
  $sql .= ", num = 1 ";
  $sql .= "ON DUPLICATE KEY UPDATE num = num + 1, last = NOW()";
  dbg("sadwl: sadwl_check: SQL: " . $sql);
  $rv = $dbh->do($sql);
  if ($rv) {
   dbg("sadwl: sadwl_check: Inserted/Updated sadwl: " . $fromdomain . " " . $envfromdomain . " " . $domreverse);
  } else {
   die "sadwl: sadwl_check: Sql Error: " . $dbh->errstr;
  }

  # Search for a record key and set score if found
  dbg("sadwl: sadwl: Search sadwl: " . $fromdomain . " " . $envfromdomain . " " . $domreverse);
  $sql = "SELECT score FROM sadwl WHERE domain = '$fromdomain' AND envdomain = '$envfromdomain' AND srv_dom_reverse = '$domreverse' ";
  $sth = $dbh->prepare($sql);
  $sth->execute();
  $vscore = $sth->fetchrow();
  $sth->finish();
  if ($vscore) {
   dbg("sadwl: sadwl_check: Score: " . $vscore);
   $pms->got_hit("SADWL", "HEADER: ", score => $vscore);
   return 1;
  } else {
   dbg("sadwl: sadwl_check: Not found or no valued score to assign");
  }

  # Disconnect DB
  $dbh->disconnect();
  dbg("sadwl: sadwl_check: Disconnected DB");

  return 0;
}

1;

